@@ -14,12 +14,13 @@ import (
 	"github.com/syncthing/syncthing/lib/fs"
 	"github.com/syncthing/syncthing/lib/protocol"
 	"github.com/syncthing/syncthing/lib/util"
+	"github.com/syncthing/syncthing/lib/versioner"
 )
 
 var (
-	errPathNotDirectory = errors.New("folder path not a directory")
-	errPathMissing      = errors.New("folder path missing")
-	errMarkerMissing    = errors.New("folder marker missing")
+	ErrPathNotDirectory = errors.New("folder path not a directory")
+	ErrPathMissing      = errors.New("folder path missing")
+	ErrMarkerMissing    = errors.New("folder marker missing")
 )
 
 const DefaultMarkerName = ".stfolder"
@@ -39,7 +40,7 @@ type FolderConfiguration struct {
 	MinDiskFree           Size                        `xml:"minDiskFree" json:"minDiskFree"`
 	Versioning            VersioningConfiguration     `xml:"versioning" json:"versioning"`
 	Copiers               int                         `xml:"copiers" json:"copiers"` // This defines how many files are handled concurrently.
-	Pullers               int                         `xml:"pullers" json:"pullers"` // Defines how many blocks are fetched at the same time, possibly between separate copier routines.
+	PullerMaxPendingKiB   int                         `xml:"pullerMaxPendingKiB" json:"pullerMaxPendingKiB"`
 	Hashers               int                         `xml:"hashers" json:"hashers"` // Less than one sets the value to the number of cores. These are CPU bound due to hashing.
 	Order                 PullOrder                   `xml:"order" json:"order"`
 	IgnoreDelete          bool                        `xml:"ignoreDelete" json:"ignoreDelete"`
@@ -51,11 +52,13 @@ type FolderConfiguration struct {
 	Paused                bool                        `xml:"paused" json:"paused"`
 	WeakHashThresholdPct  int                         `xml:"weakHashThresholdPct" json:"weakHashThresholdPct"` // Use weak hash if more than X percent of the file has changed. Set to -1 to always use weak hash.
 	MarkerName            string                      `xml:"markerName" json:"markerName"`
+	UseLargeBlocks        bool                        `xml:"useLargeBlocks" json:"useLargeBlocks"`
 
 	cachedFilesystem fs.Filesystem
 
 	DeprecatedReadOnly       bool    `xml:"ro,attr,omitempty" json:"-"`
 	DeprecatedMinDiskFreePct float64 `xml:"minDiskFreePct,omitempty" json:"-"`
+	DeprecatedPullers        int     `xml:"pullers,omitempty" json:"-"`
 }
 
 type FolderDeviceConfiguration struct {
@@ -65,16 +68,17 @@ type FolderDeviceConfiguration struct {
 
 func NewFolderConfiguration(myID protocol.DeviceID, id, label string, fsType fs.FilesystemType, path string) FolderConfiguration {
 	f := FolderConfiguration{
-		ID:              id,
-		Label:           label,
-		RescanIntervalS: 60,
-		FSWatcherDelayS: 10,
-		MinDiskFree:     Size{Value: 1, Unit: "%"},
-		Devices:         []FolderDeviceConfiguration{{DeviceID: myID}},
-		AutoNormalize:   true,
-		MaxConflicts:    -1,
-		FilesystemType:  fsType,
-		Path:            path,
+		ID:               id,
+		Label:            label,
+		RescanIntervalS:  3600,
+		FSWatcherEnabled: true,
+		FSWatcherDelayS:  10,
+		MinDiskFree:      Size{Value: 1, Unit: "%"},
+		Devices:          []FolderDeviceConfiguration{{DeviceID: myID}},
+		AutoNormalize:    true,
+		MaxConflicts:     -1,
+		FilesystemType:   fsType,
+		Path:             path,
 	}
 	f.prepare()
 	return f
@@ -98,8 +102,20 @@ func (f FolderConfiguration) Filesystem() fs.Filesystem {
 	return f.cachedFilesystem
 }
 
+func (f FolderConfiguration) Versioner() versioner.Versioner {
+	if f.Versioning.Type == "" {
+		return nil
+	}
+	versionerFactory, ok := versioner.Factories[f.Versioning.Type]
+	if !ok {
+		l.Fatalf("Requested versioning type %q that does not exist", f.Versioning.Type)
+	}
+
+	return versionerFactory(f.ID, f.Filesystem(), f.Versioning.Params)
+}
+
 func (f *FolderConfiguration) CreateMarker() error {
-	if err := f.CheckPath(); err != errMarkerMissing {
+	if err := f.CheckPath(); err != ErrMarkerMissing {
 		return err
 	}
 	if f.MarkerName != DefaultMarkerName {
@@ -137,7 +153,7 @@ func (f *FolderConfiguration) CheckPath() error {
 		if !fs.IsNotExist(err) {
 			return err
 		}
-		return errPathMissing
+		return ErrPathMissing
 	}
 
 	// Users might have the root directory as a symlink or reparse point.
@@ -147,7 +163,7 @@ func (f *FolderConfiguration) CheckPath() error {
 	// Stat ends up calling stat on C:\dir\file\ which, fails with "is not a directory"
 	// in the error check above, and we don't even get to here.
 	if !fi.IsDir() && !fi.IsSymlink() {
-		return errPathNotDirectory
+		return ErrPathNotDirectory
 	}
 
 	_, err = f.Filesystem().Stat(f.MarkerName)
@@ -155,7 +171,7 @@ func (f *FolderConfiguration) CheckPath() error {
 		if !fs.IsNotExist(err) {
 			return err
 		}
-		return errMarkerMissing
+		return ErrMarkerMissing
 	}
 
 	return nil
@@ -262,3 +278,17 @@ func (l FolderDeviceConfigurationList) Len() int {
 func (f *FolderConfiguration) CheckFreeSpace() (err error) {
 	return checkFreeSpace(f.MinDiskFree, f.Filesystem())
 }
+
+type FolderConfigurationList []FolderConfiguration
+
+func (l FolderConfigurationList) Len() int {
+	return len(l)
+}
+
+func (l FolderConfigurationList) Less(a, b int) bool {
+	return l[a].ID < l[b].ID
+}
+
+func (l FolderConfigurationList) Swap(a, b int) {
+	l[a], l[b] = l[b], l[a]
+}
