@@ -35,10 +35,13 @@ type Instance struct {
 }
 
 const (
-	keyPrefixLen = 1
-	keyFolderLen = 4 // indexed
-	keyDeviceLen = 4 // indexed
-	keyHashLen   = 32
+	keyPrefixLen   = 1
+	keyFolderLen   = 4 // indexed
+	keyDeviceLen   = 4 // indexed
+	keySequenceLen = 8
+	keyHashLen     = 32
+
+	maxInt64 int64 = 1<<63 - 1
 )
 
 func Open(file string) (*Instance, error) {
@@ -83,6 +86,28 @@ func newDBInstance(db *leveldb.DB, location string) *Instance {
 	return i
 }
 
+// UpdateSchema does transitions to the current db version if necessary
+func (db *Instance) UpdateSchema() {
+	miscDB := NewNamespacedKV(db, string(KeyTypeMiscData))
+	prevVersion, _ := miscDB.Int64("dbVersion")
+
+	if prevVersion >= dbVersion {
+		return
+	}
+
+	l.Infof("Updating database schema version from %v to %v...", prevVersion, dbVersion)
+
+	if prevVersion == 0 {
+		db.updateSchema0to1()
+	}
+	if prevVersion <= 1 {
+		db.updateSchema1to2()
+	}
+
+	l.Infof("Finished updating database schema version from %v to %v", prevVersion, dbVersion)
+	miscDB.PutInt64("dbVersion", dbVersion)
+}
+
 // Committed returns the number of items committed to the database since startup
 func (db *Instance) Committed() int64 {
 	return atomic.LoadInt64(&db.committed)
@@ -98,9 +123,10 @@ func (db *Instance) updateFiles(folder, device []byte, fs []protocol.FileInfo, m
 	defer t.close()
 
 	var fk []byte
+	var gk []byte
 	for _, f := range fs {
 		name := []byte(f.Name)
-		fk = db.deviceKeyInto(fk[:cap(fk)], folder, device, name)
+		fk = db.deviceKeyInto(fk, folder, device, name)
 
 		// Get and unmarshal the file entry. If it doesn't exist or can't be
 		// unmarshalled we'll add it as a new entry.
@@ -121,8 +147,10 @@ func (db *Instance) updateFiles(folder, device []byte, fs []protocol.FileInfo, m
 		}
 		meta.addFile(devID, f)
 
-		t.insertFile(folder, device, f)
-		t.updateGlobal(folder, device, f, meta)
+		t.insertFile(fk, folder, device, f)
+
+		gk = db.globalKeyInto(gk, folder, name)
+		t.updateGlobal(gk, folder, device, f, meta)
 
 		// Write out and reuse the batch every few records, to avoid the batch
 		// growing too large and thus allocating unnecessarily much memory.
@@ -130,6 +158,33 @@ func (db *Instance) updateFiles(folder, device []byte, fs []protocol.FileInfo, m
 	}
 }
 
+func (db *Instance) addSequences(folder []byte, fs []protocol.FileInfo) {
+	t := db.newReadWriteTransaction()
+	defer t.close()
+
+	var sk []byte
+	var dk []byte
+	for _, f := range fs {
+		sk = db.sequenceKeyInto(sk, folder, f.Sequence)
+		dk = db.deviceKeyInto(dk, folder, protocol.LocalDeviceID[:], []byte(f.Name))
+		t.Put(sk, dk)
+		l.Debugf("adding sequence; folder=%q sequence=%v %v", folder, f.Sequence, f.Name)
+		t.checkFlush()
+	}
+}
+
+func (db *Instance) removeSequences(folder []byte, fs []protocol.FileInfo) {
+	t := db.newReadWriteTransaction()
+	defer t.close()
+
+	var sk []byte
+	for _, f := range fs {
+		t.Delete(db.sequenceKeyInto(sk, folder, f.Sequence))
+		l.Debugf("removing sequence; folder=%q sequence=%v %v", folder, f.Sequence, f.Name)
+		t.checkFlush()
+	}
+}
+
 func (db *Instance) withHave(folder, device, prefix []byte, truncate bool, fn Iterator) {
 	t := db.newReadOnlyTransaction()
 	defer t.close()
@@ -157,7 +212,26 @@ func (db *Instance) withHave(folder, device, prefix []byte, truncate bool, fn It
 			l.Debugln("unmarshal error:", err)
 			continue
 		}
-		if cont := fn(f); !cont {
+		if !fn(f) {
+			return
+		}
+	}
+}
+
+func (db *Instance) withHaveSequence(folder []byte, startSeq int64, fn Iterator) {
+	t := db.newReadOnlyTransaction()
+	defer t.close()
+
+	dbi := t.NewIterator(&util.Range{Start: db.sequenceKey(folder, startSeq), Limit: db.sequenceKey(folder, maxInt64)}, nil)
+	defer dbi.Release()
+
+	for dbi.Next() {
+		f, ok := db.getFile(dbi.Value())
+		if !ok {
+			l.Debugln("missing file for sequence number", db.sequenceKeySequence(dbi.Key()))
+			continue
+		}
+		if !fn(f) {
 			return
 		}
 	}
@@ -170,6 +244,8 @@ func (db *Instance) withAllFolderTruncated(folder []byte, fn func(device []byte,
 	dbi := t.NewIterator(util.BytesPrefix(db.deviceKey(folder, nil, nil)[:keyPrefixLen+keyFolderLen]), nil)
 	defer dbi.Release()
 
+	var gk []byte
+
 	for dbi.Next() {
 		device := db.deviceKeyDevice(dbi.Key())
 		var f FileInfoTruncated
@@ -186,20 +262,37 @@ func (db *Instance) withAllFolderTruncated(folder []byte, fn func(device []byte,
 		switch f.Name {
 		case "", ".", "..", "/": // A few obviously invalid filenames
 			l.Infof("Dropping invalid filename %q from database", f.Name)
-			t.removeFromGlobal(folder, device, nil, nil)
+			name := []byte(f.Name)
+			gk = db.globalKeyInto(gk, folder, name)
+			t.removeFromGlobal(gk, folder, device, name, nil)
 			t.Delete(dbi.Key())
 			t.checkFlush()
 			continue
 		}
 
-		if cont := fn(device, f); !cont {
+		if !fn(device, f) {
 			return
 		}
 	}
 }
 
-func (db *Instance) getFile(folder, device, file []byte) (protocol.FileInfo, bool) {
-	return getFile(db, db.deviceKey(folder, device, file))
+func (db *Instance) getFile(key []byte) (protocol.FileInfo, bool) {
+	bs, err := db.Get(key, nil)
+	if err == leveldb.ErrNotFound {
+		return protocol.FileInfo{}, false
+	}
+	if err != nil {
+		l.Debugln("surprise error:", err)
+		return protocol.FileInfo{}, false
+	}
+
+	var f protocol.FileInfo
+	err = f.Unmarshal(bs)
+	if err != nil {
+		l.Debugln("unmarshal error:", err)
+		return protocol.FileInfo{}, false
+	}
+	return f, true
 }
 
 func (db *Instance) getGlobal(folder, file []byte, truncate bool) (FileIntf, bool) {
@@ -272,7 +365,7 @@ func (db *Instance) withGlobal(folder, prefix []byte, truncate bool, fn Iterator
 			return
 		}
 
-		fk = db.deviceKeyInto(fk[:cap(fk)], folder, vl.Versions[0].Device, name)
+		fk = db.deviceKeyInto(fk, folder, vl.Versions[0].Device, name)
 		bs, err := t.Get(fk, nil)
 		if err != nil {
 			l.Debugln("surprise error:", err)
@@ -285,7 +378,7 @@ func (db *Instance) withGlobal(folder, prefix []byte, truncate bool, fn Iterator
 			continue
 		}
 
-		if cont := fn(f); !cont {
+		if !fn(f) {
 			return
 		}
 	}
@@ -324,7 +417,7 @@ func (db *Instance) availability(folder, file []byte) []protocol.DeviceID {
 	return devices
 }
 
-func (db *Instance) withNeed(folder, device []byte, truncate bool, needAllInvalid bool, fn Iterator) {
+func (db *Instance) withNeed(folder, device []byte, truncate bool, fn Iterator) {
 	t := db.newReadOnlyTransaction()
 	defer t.close()
 
@@ -351,12 +444,6 @@ func (db *Instance) withNeed(folder, device []byte, truncate bool, needAllInvali
 			if bytes.Equal(v.Device, device) {
 				have = true
 				haveFileVersion = v
-				// We need invalid files regardless of version when
-				// ignore patterns changed
-				if v.Invalid && needAllInvalid {
-					need = true
-					break
-				}
 				// XXX: This marks Concurrent (i.e. conflicting) changes as
 				// needs. Maybe we should do that, but it needs special
 				// handling in the puller.
@@ -384,7 +471,7 @@ func (db *Instance) withNeed(folder, device []byte, truncate bool, needAllInvali
 				continue
 			}
 
-			fk = db.deviceKeyInto(fk[:cap(fk)], folder, vl.Versions[i].Device, name)
+			fk = db.deviceKeyInto(fk, folder, vl.Versions[i].Device, name)
 			bs, err := t.Get(fk, nil)
 			if err != nil {
 				l.Debugln("surprise error:", err)
@@ -404,7 +491,7 @@ func (db *Instance) withNeed(folder, device []byte, truncate bool, needAllInvali
 
 			l.Debugf("need folder=%q device=%v name=%q need=%v have=%v invalid=%v haveV=%v globalV=%v globalDev=%v", folder, protocol.DeviceIDFromBytes(device), name, need, have, haveFileVersion.Invalid, haveFileVersion.Version, needVersion, needDevice)
 
-			if cont := fn(gf); !cont {
+			if !fn(gf) {
 				return
 			}
 
@@ -452,6 +539,14 @@ func (db *Instance) dropFolder(folder []byte) {
 	}
 	dbi.Release()
 
+	// Remove all sequences related to the folder
+	sequenceKey := db.sequenceKey([]byte(folder), 0)
+	dbi = t.NewIterator(util.BytesPrefix(sequenceKey[:keyPrefixLen+keyFolderLen]), nil)
+	for dbi.Next() {
+		db.Delete(dbi.Key(), nil)
+	}
+	dbi.Release()
+
 	// Remove all items related to the given folder from the global bucket
 	dbi = t.NewIterator(util.BytesPrefix([]byte{KeyTypeGlobal}), nil)
 	for dbi.Next() {
@@ -470,10 +565,13 @@ func (db *Instance) dropDeviceFolder(device, folder []byte, meta *metadataTracke
 	dbi := t.NewIterator(util.BytesPrefix(db.deviceKey(folder, device, nil)), nil)
 	defer dbi.Release()
 
+	var gk []byte
+
 	for dbi.Next() {
 		key := dbi.Key()
 		name := db.deviceKeyName(key)
-		t.removeFromGlobal(folder, device, name, meta)
+		gk = db.globalKeyInto(gk, folder, name)
+		t.removeFromGlobal(gk, folder, device, name, meta)
 		t.Delete(key)
 		t.checkFlush()
 	}
@@ -504,8 +602,7 @@ func (db *Instance) checkGlobals(folder []byte, meta *metadataTracker) {
 		name := db.globalKeyName(gk)
 		var newVL VersionList
 		for i, version := range vl.Versions {
-			fk = db.deviceKeyInto(fk[:cap(fk)], folder, version.Device, name)
-
+			fk = db.deviceKeyInto(fk, folder, version.Device, name)
 			_, err := t.Get(fk, nil)
 			if err == leveldb.ErrNotFound {
 				continue
@@ -517,7 +614,7 @@ func (db *Instance) checkGlobals(folder []byte, meta *metadataTracker) {
 			newVL.Versions = append(newVL.Versions, version)
 
 			if i == 0 {
-				if fi, ok := t.getFile(folder, version.Device, name); ok {
+				if fi, ok := db.getFile(fk); ok {
 					meta.addFile(globalDeviceID, fi)
 				}
 			}
@@ -531,21 +628,41 @@ func (db *Instance) checkGlobals(folder []byte, meta *metadataTracker) {
 	l.Debugf("db check completed for %q", folder)
 }
 
-// ConvertSymlinkTypes should be run once only on an old database. It
-// changes SYMLINK_FILE and SYMLINK_DIRECTORY types to the current SYMLINK
-// type (previously SYMLINK_UNKNOWN). It does this for all devices, both
-// local and remote, and does not reset delta indexes. It shouldn't really
-// matter what the symlink type is, but this cleans it up for a possible
-// future when SYMLINK_FILE and SYMLINK_DIRECTORY are no longer understood.
-func (db *Instance) ConvertSymlinkTypes() {
+func (db *Instance) updateSchema0to1() {
 	t := db.newReadWriteTransaction()
 	defer t.close()
 
 	dbi := t.NewIterator(util.BytesPrefix([]byte{KeyTypeDevice}), nil)
 	defer dbi.Release()
 
-	conv := 0
+	symlinkConv := 0
+	changedFolders := make(map[string]struct{})
+	ignAdded := 0
+	meta := newMetadataTracker() // dummy metadata tracker
+	var gk []byte
+
 	for dbi.Next() {
+		folder := db.deviceKeyFolder(dbi.Key())
+		device := db.deviceKeyDevice(dbi.Key())
+		name := db.deviceKeyName(dbi.Key())
+
+		// Remove files with absolute path (see #4799)
+		if strings.HasPrefix(string(name), "/") {
+			if _, ok := changedFolders[string(folder)]; !ok {
+				changedFolders[string(folder)] = struct{}{}
+			}
+			gk = db.globalKeyInto(gk, folder, name)
+			t.removeFromGlobal(gk, folder, device, nil, nil)
+			t.Delete(dbi.Key())
+			t.checkFlush()
+			continue
+		}
+
+		// Change SYMLINK_FILE and SYMLINK_DIRECTORY types to the current SYMLINK
+		// type (previously SYMLINK_UNKNOWN). It does this for all devices, both
+		// local and remote, and does not reset delta indexes. It shouldn't really
+		// matter what the symlink type is, but this cleans it up for a possible
+		// future when SYMLINK_FILE and SYMLINK_DIRECTORY are no longer understood.
 		var f protocol.FileInfo
 		if err := f.Unmarshal(dbi.Value()); err != nil {
 			// probably can't happen
@@ -559,99 +676,45 @@ func (db *Instance) ConvertSymlinkTypes() {
 			}
 			t.Put(dbi.Key(), bs)
 			t.checkFlush()
-			conv++
+			symlinkConv++
+		}
+
+		// Add invalid files to global list
+		if f.Invalid {
+			gk = db.globalKeyInto(gk, folder, name)
+			if t.updateGlobal(gk, folder, device, f, meta) {
+				if _, ok := changedFolders[string(folder)]; !ok {
+					changedFolders[string(folder)] = struct{}{}
+				}
+				ignAdded++
+			}
 		}
 	}
 
-	l.Infof("Updated symlink type for %d index entries", conv)
+	for folder := range changedFolders {
+		db.dropFolderMeta([]byte(folder))
+	}
+
+	l.Infof("Updated symlink type for %d index entries and added %d invalid files to global list", symlinkConv, ignAdded)
 }
 
-// AddInvalidToGlobal searches for invalid files and adds them to the global list.
-// Invalid files exist in the db if they once were not ignored and subsequently
-// ignored. In the new system this is still valid, but invalid files must also be
-// in the global list such that they cannot be mistaken for missing files.
-func (db *Instance) AddInvalidToGlobal(folder, device []byte) int {
+func (db *Instance) updateSchema1to2() {
 	t := db.newReadWriteTransaction()
 	defer t.close()
 
-	dbi := t.NewIterator(util.BytesPrefix(db.deviceKey(folder, device, nil)[:keyPrefixLen+keyFolderLen+keyDeviceLen]), nil)
-	defer dbi.Release()
+	var sk []byte
+	var dk []byte
 
-	changed := 0
-	for dbi.Next() {
-		var file protocol.FileInfo
-		if err := file.Unmarshal(dbi.Value()); err != nil {
-			// probably can't happen
-			continue
-		}
-		if file.Invalid {
-			changed++
-
-			l.Debugf("add invalid to global; folder=%q device=%v file=%q version=%v", folder, protocol.DeviceIDFromBytes(device), file.Name, file.Version)
-
-			// this is an adapted version of readWriteTransaction.updateGlobal
-			name := []byte(file.Name)
-			gk := t.db.globalKey(folder, name)
-
-			var fl VersionList
-			if svl, err := t.Get(gk, nil); err == nil {
-				fl.Unmarshal(svl) // skip error, range handles success case
-			}
-
-			nv := FileVersion{
-				Device:  device,
-				Version: file.Version,
-				Invalid: file.Invalid,
-			}
-
-			inserted := false
-			// Find a position in the list to insert this file. The file at the front
-			// of the list is the newer, the "global".
-		insert:
-			for i := range fl.Versions {
-				switch fl.Versions[i].Version.Compare(file.Version) {
-				case protocol.Equal:
-					// Invalid files should go after a valid file of equal version
-					if nv.Invalid {
-						continue insert
-					}
-					fallthrough
-
-				case protocol.Lesser:
-					// The version at this point in the list is equal to or lesser
-					// ("older") than us. We insert ourselves in front of it.
-					fl.Versions = insertVersion(fl.Versions, i, nv)
-					inserted = true
-					break insert
-
-				case protocol.ConcurrentLesser, protocol.ConcurrentGreater:
-					// The version at this point is in conflict with us. We must pull
-					// the actual file metadata to determine who wins. If we win, we
-					// insert ourselves in front of the loser here. (The "Lesser" and
-					// "Greater" in the condition above is just based on the device
-					// IDs in the version vector, which is not the only thing we use
-					// to determine the winner.)
-					//
-					// A surprise missing file entry here is counted as a win for us.
-					of, ok := t.getFile(folder, fl.Versions[i].Device, name)
-					if !ok || file.WinsConflict(of) {
-						fl.Versions = insertVersion(fl.Versions, i, nv)
-						inserted = true
-						break insert
-					}
-				}
-			}
-
-			if !inserted {
-				// We didn't find a position for an insert above, so append to the end.
-				fl.Versions = append(fl.Versions, nv)
-			}
-
-			t.Put(gk, mustMarshal(&fl))
-		}
+	for _, folderStr := range db.ListFolders() {
+		folder := []byte(folderStr)
+		db.withHave(folder, protocol.LocalDeviceID[:], nil, true, func(f FileIntf) bool {
+			sk = db.sequenceKeyInto(sk, folder, f.SequenceNo())
+			dk = db.deviceKeyInto(dk, folder, protocol.LocalDeviceID[:], []byte(f.FileName()))
+			t.Put(sk, dk)
+			t.checkFlush()
+			return true
+		})
 	}
-
-	return changed
 }
 
 // deviceKey returns a byte slice encoding the following information:
@@ -663,16 +726,14 @@ func (db *Instance) deviceKey(folder, device, file []byte) []byte {
 	return db.deviceKeyInto(nil, folder, device, file)
 }
 
-func (db *Instance) deviceKeyInto(k []byte, folder, device, file []byte) []byte {
+func (db *Instance) deviceKeyInto(k, folder, device, file []byte) []byte {
 	reqLen := keyPrefixLen + keyFolderLen + keyDeviceLen + len(file)
-	if len(k) < reqLen {
-		k = make([]byte, reqLen)
-	}
+	k = resize(k, reqLen)
 	k[0] = KeyTypeDevice
 	binary.BigEndian.PutUint32(k[keyPrefixLen:], db.folderIdx.ID(folder))
 	binary.BigEndian.PutUint32(k[keyPrefixLen+keyFolderLen:], db.deviceIdx.ID(device))
 	copy(k[keyPrefixLen+keyFolderLen+keyDeviceLen:], file)
-	return k[:reqLen]
+	return k
 }
 
 // deviceKeyName returns the device ID from the key
@@ -703,11 +764,16 @@ func (db *Instance) deviceKeyDevice(key []byte) []byte {
 //	   folder (4 bytes)
 //	   name (variable size)
 func (db *Instance) globalKey(folder, file []byte) []byte {
-	k := make([]byte, keyPrefixLen+keyFolderLen+len(file))
-	k[0] = KeyTypeGlobal
-	binary.BigEndian.PutUint32(k[keyPrefixLen:], db.folderIdx.ID(folder))
-	copy(k[keyPrefixLen+keyFolderLen:], file)
-	return k
+	return db.globalKeyInto(nil, folder, file)
+}
+
+func (db *Instance) globalKeyInto(gk, folder, file []byte) []byte {
+	reqLen := keyPrefixLen + keyFolderLen + len(file)
+	gk = resize(gk, reqLen)
+	gk[0] = KeyTypeGlobal
+	binary.BigEndian.PutUint32(gk[keyPrefixLen:], db.folderIdx.ID(folder))
+	copy(gk[keyPrefixLen+keyFolderLen:], file)
+	return gk[:reqLen]
 }
 
 // globalKeyName returns the filename from the key
@@ -720,6 +786,28 @@ func (db *Instance) globalKeyFolder(key []byte) ([]byte, bool) {
 	return db.folderIdx.Val(binary.BigEndian.Uint32(key[keyPrefixLen:]))
 }
 
+// sequenceKey returns a byte slice encoding the following information:
+//	   KeyTypeSequence (1 byte)
+//	   folder (4 bytes)
+//	   sequence number (8 bytes)
+func (db *Instance) sequenceKey(folder []byte, seq int64) []byte {
+	return db.sequenceKeyInto(nil, folder, seq)
+}
+
+func (db *Instance) sequenceKeyInto(k []byte, folder []byte, seq int64) []byte {
+	reqLen := keyPrefixLen + keyFolderLen + keySequenceLen
+	k = resize(k, reqLen)
+	k[0] = KeyTypeSequence
+	binary.BigEndian.PutUint32(k[keyPrefixLen:], db.folderIdx.ID(folder))
+	binary.BigEndian.PutUint64(k[keyPrefixLen+keyFolderLen:], uint64(seq))
+	return k[:reqLen]
+}
+
+// sequenceKeySequence returns the sequence number from the key
+func (db *Instance) sequenceKeySequence(key []byte) int64 {
+	return int64(binary.BigEndian.Uint64(key[keyPrefixLen+keyFolderLen:]))
+}
+
 func (db *Instance) getIndexID(device, folder []byte) protocol.IndexID {
 	key := db.indexIDKey(device, folder)
 	cur, err := db.Get(key, nil)
@@ -751,6 +839,15 @@ func (db *Instance) indexIDKey(device, folder []byte) []byte {
 	return k
 }
 
+func (db *Instance) indexIDDevice(key []byte) []byte {
+	device, ok := db.deviceIdx.Val(binary.BigEndian.Uint32(key[keyPrefixLen:]))
+	if !ok {
+		// uuh ...
+		return nil
+	}
+	return device
+}
+
 func (db *Instance) mtimesKey(folder []byte) []byte {
 	prefix := make([]byte, 5) // key type + 4 bytes folder idx number
 	prefix[0] = KeyTypeVirtualMtime
@@ -765,10 +862,33 @@ func (db *Instance) folderMetaKey(folder []byte) []byte {
 	return prefix
 }
 
-// DropDeltaIndexIDs removes all index IDs from the database. This will
-// cause a full index transmission on the next connection.
-func (db *Instance) DropDeltaIndexIDs() {
-	db.dropPrefix([]byte{KeyTypeIndexID})
+// DropLocalDeltaIndexIDs removes all index IDs for the local device ID from
+// the database. This will cause a full index transmission on the next
+// connection.
+func (db *Instance) DropLocalDeltaIndexIDs() {
+	db.dropDeltaIndexIDs(true)
+}
+
+// DropRemoteDeltaIndexIDs removes all index IDs for the other devices than
+// the local one from the database. This will cause them to send us a full
+// index on the next connection.
+func (db *Instance) DropRemoteDeltaIndexIDs() {
+	db.dropDeltaIndexIDs(false)
+}
+
+func (db *Instance) dropDeltaIndexIDs(local bool) {
+	t := db.newReadWriteTransaction()
+	defer t.close()
+
+	dbi := t.NewIterator(util.BytesPrefix([]byte{KeyTypeIndexID}), nil)
+	defer dbi.Release()
+
+	for dbi.Next() {
+		device := db.indexIDDevice(dbi.Key())
+		if bytes.Equal(device, protocol.LocalDeviceID[:]) == local {
+			t.Delete(dbi.Key())
+		}
+	}
 }
 
 func (db *Instance) dropMtimes(folder []byte) {
@@ -903,3 +1023,11 @@ func (i *smallIndex) Val(id uint32) ([]byte, bool) {
 
 	return []byte(val), true
 }
+
+// resize returns a byte array of length reqLen, reusing k if possible
+func resize(k []byte, reqLen int) []byte {
+	if cap(k) < reqLen {
+		return make([]byte, reqLen)
+	}
+	return k[:reqLen]
+}
