@@ -43,6 +43,7 @@ type FileIntf interface {
 	IsSymlink() bool
 	HasPermissionBits() bool
 	SequenceNo() int64
+	BlockSize() int
 }
 
 // The Iterator is called with either a protocol.FileInfo or a
@@ -124,6 +125,10 @@ func (s *FileSet) Drop(device protocol.DeviceID) {
 
 func (s *FileSet) Update(device protocol.DeviceID, fs []protocol.FileInfo) {
 	l.Debugf("%s Update(%v, [%d])", s.folder, device, len(fs))
+
+	// do not modify fs in place, it is still used in outer scope
+	fs = append([]protocol.FileInfo(nil), fs...)
+
 	normalizeFilenames(fs)
 
 	s.updateMutex.Lock()
@@ -136,8 +141,11 @@ func (s *FileSet) Update(device protocol.DeviceID, fs []protocol.FileInfo) {
 		// filter slice according to https://github.com/golang/go/wiki/SliceTricks#filtering-without-allocating
 		oldFs := fs
 		fs = fs[:0]
+		var dk []byte
+		folder := []byte(s.folder)
 		for _, nf := range oldFs {
-			ef, ok := s.db.getFile([]byte(s.folder), device[:], []byte(nf.Name))
+			dk = s.db.deviceKeyInto(dk, folder, device[:], []byte(osutil.NormalizedFilename(nf.Name)))
+			ef, ok := s.db.getFile(dk)
 			if ok && ef.Version.Equal(nf.Version) && ef.Invalid == nf.Invalid {
 				continue
 			}
@@ -152,6 +160,8 @@ func (s *FileSet) Update(device protocol.DeviceID, fs []protocol.FileInfo) {
 		}
 		s.blockmap.Discard(discards)
 		s.blockmap.Update(updates)
+		s.db.removeSequences(folder, discards)
+		s.db.addSequences(folder, updates)
 	}
 
 	s.db.updateFiles([]byte(s.folder), device[:], fs, s.meta)
@@ -160,24 +170,12 @@ func (s *FileSet) Update(device protocol.DeviceID, fs []protocol.FileInfo) {
 
 func (s *FileSet) WithNeed(device protocol.DeviceID, fn Iterator) {
 	l.Debugf("%s WithNeed(%v)", s.folder, device)
-	s.db.withNeed([]byte(s.folder), device[:], false, false, nativeFileIterator(fn))
+	s.db.withNeed([]byte(s.folder), device[:], false, nativeFileIterator(fn))
 }
 
 func (s *FileSet) WithNeedTruncated(device protocol.DeviceID, fn Iterator) {
 	l.Debugf("%s WithNeedTruncated(%v)", s.folder, device)
-	s.db.withNeed([]byte(s.folder), device[:], true, false, nativeFileIterator(fn))
-}
-
-// WithNeedOrInvalid considers all invalid files as needed, regardless of their version
-// (e.g. for pulling when ignore patterns changed)
-func (s *FileSet) WithNeedOrInvalid(device protocol.DeviceID, fn Iterator) {
-	l.Debugf("%s WithNeedExcludingInvalid(%v)", s.folder, device)
-	s.db.withNeed([]byte(s.folder), device[:], false, true, nativeFileIterator(fn))
-}
-
-func (s *FileSet) WithNeedOrInvalidTruncated(device protocol.DeviceID, fn Iterator) {
-	l.Debugf("%s WithNeedExcludingInvalidTruncated(%v)", s.folder, device)
-	s.db.withNeed([]byte(s.folder), device[:], true, true, nativeFileIterator(fn))
+	s.db.withNeed([]byte(s.folder), device[:], true, nativeFileIterator(fn))
 }
 
 func (s *FileSet) WithHave(device protocol.DeviceID, fn Iterator) {
@@ -190,6 +188,11 @@ func (s *FileSet) WithHaveTruncated(device protocol.DeviceID, fn Iterator) {
 	s.db.withHave([]byte(s.folder), device[:], nil, true, nativeFileIterator(fn))
 }
 
+func (s *FileSet) WithHaveSequence(startSeq int64, fn Iterator) {
+	l.Debugf("%s WithHaveSequence(%v)", s.folder, startSeq)
+	s.db.withHaveSequence([]byte(s.folder), startSeq, nativeFileIterator(fn))
+}
+
 func (s *FileSet) WithPrefixedHaveTruncated(device protocol.DeviceID, prefix string, fn Iterator) {
 	l.Debugf("%s WithPrefixedHaveTruncated(%v)", s.folder, device)
 	s.db.withHave([]byte(s.folder), device[:], []byte(osutil.NormalizedFilename(prefix)), true, nativeFileIterator(fn))
@@ -210,7 +213,7 @@ func (s *FileSet) WithPrefixedGlobalTruncated(prefix string, fn Iterator) {
 }
 
 func (s *FileSet) Get(device protocol.DeviceID, file string) (protocol.FileInfo, bool) {
-	f, ok := s.db.getFile([]byte(s.folder), device[:], []byte(osutil.NormalizedFilename(file)))
+	f, ok := s.db.getFile(s.db.deviceKey([]byte(s.folder), device[:], []byte(osutil.NormalizedFilename(file))))
 	f.Name = osutil.NativeFilename(f.Name)
 	return f, ok
 }
