@@ -8,7 +8,7 @@
 
 package fs
 
-import "github.com/Zillode/notify"
+import "github.com/syncthing/notify"
 
 const (
 	subEventMask  = notify.FileNotifyChangeFileName | notify.FileNotifyChangeDirName | notify.FileNotifyChangeSize | notify.FileNotifyChangeCreation
